/* New World Order 1.0                                                        */
/* A preprocessor for CircleMUD 3.0 beta                                      */
/* Copyright 1996 by Mark Hughes <kamikaze@kuoi.asui.uidaho.edu>              */
/* All rights reserved.                                                       */
/* You may freely redistribute this software, as long as it is unchanged.  No */
/* modified version of this software may be distributed, and it must always   */
/* retain this copyright notice and the accompanying documentation.  You may  */
/* not charge for the distribution of this software, even to cover disk &     */
/* copying charges.  These terms are only subject to change with the express  */
/* WRITTEN permission of the author.                                          */
/* The author is not responsible for any data loss or damage caused by this   */
/* program.  Backups are your friend.                                         */

debug = 0
system = "os2"

parse arg zone .

if zone="" | \isint(zone) then do
  say "Usage: nwo zone-number"
  exit -1
end /* if */

/* globals */
nwofile = zone".nwo"
zonfile = zone".zon"
wldfile = zone".wld"
objfile = zone".obj"
mobfile = zone".mob"
shpfile = zone".shp"

/* newlines - internally, lf is used for a newline.  The OS-dependant newline */
/* is only used for output. */
cr = d2c(13);  lf = d2c(10)

if system="os2" then              newline = cr||lf
else if system="regina/unix" then newline = lf

maxlevel=33
argv. = ""                   /* used by splitline */
argc = 0

/* experience */
xp_table. = 0
xp_table.1 = 2000
xp_table.2 = 4000
xp_table.3 = 8000
xp_table.4 = 16000
xp_table.5 = 32000
xp_table.6 = 64000
xp_table.7 = 125000
xp_table.8 = 250000
xp_table.9 = 500000
xp_table.10 = 750000
xp_table.11 = 1000000
xp_table.12 = 1250000
xp_table.13 = 1500000
xp_table.14 = 1850000
xp_table.15 = 2200000
xp_table.16 = 2550000
xp_table.17 = 2900000
xp_table.18 = 3250000
xp_table.19 = 3600000
xp_table.20 = 3900000
xp_table.21 = 4200000
xp_table.22 = 4500000
xp_table.23 = 4800000
xp_table.24 = 5150000
xp_table.25 = 5500000
xp_table.26 = 5950000
xp_table.27 = 6400000
xp_table.28 = 6850000
xp_table.29 = 7400000
xp_table.30 = 8000000
xp_table.31 = 9000000
xp_table.32 = 9500000
xp_table.33 = 10000000

/* spell list */
spell.1="armor"
spell.2="teleport"
spell.3="bless"
spell.4="blindness"
spell.5="burning hands"
spell.6="call lightning"
spell.7="charm"
spell.8="chill touch"
spell.9="clone"
spell.10="color spray"
spell.11="control weather"
spell.12="create food"
spell.13="create water"
spell.14="cure blind"
spell.15="cure critic"
spell.16="cure light"
spell.17="curse"
spell.18="detect align"
spell.19="detect invis"
spell.20="detect magic"
spell.21="detect poison"
spell.22="dispel evil"
spell.23="earthquake"
spell.24="enchant weapon"
spell.25="energy drain"
spell.26="fireball"
spell.27="harm"
spell.28="heal"
spell.29="invisible"
spell.30="lightning bolt"
spell.31="locate object"
spell.32="magic missile"
spell.33="poison"
spell.34="prot from evil"
spell.35="remove curse"
spell.36="sanctuary"
spell.37="shocking grasp"
spell.38="sleep"
spell.39="strength"
spell.40="summon"
spell.41="ventriloquate"
spell.42="word of recall"
spell.43="remove poison"
spell.44="sense life"
spell.45="animate dead"
spell.46="dispel good"
spell.47="group armor"
spell.48="group heal"
spell.49="group recall"
spell.50="infravision"
spell.51="waterwalk"
spell.201="identify"
spell.202="fire breath"
spell.203="gas breath"
spell.204="frost breath"
spell.205="acid breath"
spell.206="lightning breath"

/* open nwo file */
if \fexists(nwofile) then do
  say nwofile "does not exist"
  exit -1
end /* if */
rc=fopen(nwofile, "r")

/* back up old zone info */
if system="os2" then do
  if fexists(zonfile) then 'move' zonfile zonfile"~"
  if fexists(wldfile) then 'move' wldfile wldfile"~"
  if fexists(mobfile) then 'move' mobfile mobfile"~"
  if fexists(objfile) then 'move' objfile objfile"~"
  if fexists(shpfile) then 'move' shpfile shpfile"~"
end /* if OS/2 */
else if system="regina/unix" then do
  if fexists(zonfile) then 'mv' zonfile zonfile"~"
  if fexists(wldfile) then 'mv' wldfile wldfile"~"
  if fexists(mobfile) then 'mv' mobfile mobfile"~"
  if fexists(objfile) then 'mv' objfile objfile"~"
  if fexists(shpfile) then 'mv' shpfile shpfile"~"
end /* if regina/unix */

lineno = 0

do while lines(nwofile)<>0
  line = getline()
  if line="" then iterate

  select
    when word(line, 1)="zone" then rc=do_zone(line)
    when word(line, 1)="room" then rc=do_room(line)
    when word(line, 1)="mob" then rc=do_mob(line)
    when word(line, 1)="obj" then rc=do_obj(line)
    when word(line, 1)="shop" then rc=do_shop(line)
    when word(line, 1)="maxlevel" then do
      if isint(word(line, 2)) & word(line, 2)>=1 then
        maxlevel=word(line, 2)
      else rc=terminate("Illegal maxlevel value" word(line, 2))
    end /* maxmoblevel */

    otherwise
      rc=terminate("Illegal top-level command")
  end /* select */
end /* while lines */

/* terminate all data files */
if fexists(zonfile) then do
  rc=fopen(zonfile, "a")
  rc=fprint(zonfile, "S")
  rc=fprint(zonfile, "$")
  rc=fclose(zonfile)
end /* if */

if fexists(wldfile) then do
  rc=fopen(wldfile, "a")
  rc=fprint(wldfile, "$")
  rc=fclose(wldfile)
end /* if */

if fexists(mobfile) then do
  rc=fopen(mobfile, "a")
  rc=fprint(mobfile, "$")
  rc=fclose(mobfile)
end /* if */

if fexists(objfile) then do
  rc=fopen(objfile, "a")
  rc=fprint(objfile, "$")
  rc=fclose(objfile)
end /* if */

if fexists(shpfile) then do
  rc=fopen(shpfile, "a")
  rc=fprint(shpfile, "$~")
  rc=fclose(shpfile)
end /* if */

exit 0


/* ------------------------------------------------------------------------- */
/* do_zone(line) */
do_zone: procedure expose argc argv. nwofile zonfile lineno zone debug cr lf newline system
/* local n zone. line if_flag start i j */

  line=arg(1)
  if words(line)>1 then rc=terminate("Illegal zone command")

  zone. = ""
  zone.name = "zone "zone
  zone.lifespan = 1440
  zone.reset = 2
  zone.ncmds = 0             /* zone.cmd<n> holds the processed command */

  do until line<>"" | lines(nwofile)=0
    line = getline()
  end /* do */
  rc=splitline(line)

  do while lines(nwofile)<>0 & line<>"endzone"
    if_flag = 0

    if argv.1="name" then zone.name = argv.2

    else if argv.1="lifespan" then do
      if isint(argv.2) then zone.lifespan = argv.2
      else rc=terminate("Illegal lifespan" argv.2)
    end /* if */

    else if argv.1="reset" then do
      select
        when argv.2="never" then zone.reset=0
        when argv.2="empty" then zone.reset=1
        when argv.2="auto" then zone.reset=2
        otherwise
          rc=terminate("Illegal reset mode" argv.2)
      end  /* select */
    end /* if */

    else do
        if argv.1="then" then do
          if_flag = 1
          start = 2
        end /* if "then" */
        else start = 1

        select /* zone command */

          when argv.start="mob" then do
            zone.ncmds = zone.ncmds + 1
            key = "cmd"zone.ncmds
            i = start + 1
            if vnumtype(argv.i)="m" then zone.key = "M "if_flag" "vnum(argv.i)
            else rc=terminate("Illegal mob number" argv.i)

            i = i + 1
            if argv.i="max" then do
              i = i + 1
              if isint(argv.i) then zone.key = zone.key" "argv.i
              else rc=terminate("Illegal max number" argv.i)
              i = i + 1
            end /* if */
            else
              zone.key = zone.key" 1"
            
            if argv.i<>"in" then rc=terminate("Unexpected word" argv.i)

            i = i + 1
            if vnumtype(argv.i)='r' then zone.key = zone.key" "vnum(argv.i)
            else rc=terminate("Illegal room number" argv.i)
          end /* when mob */

          when argv.start="obj" then do
            zone.ncmds = zone.ncmds + 1
            key = "cmd"zone.ncmds
            i = start + 1
            if vnumtype(argv.i)='o' then zone.key = "O "if_flag" "vnum(argv.i)
            else rc=terminate("Illegal object number" argv.i)

            i = i + 1
            if argv.i="max" then do
              i = i + 1
              zone.key = zone.key" "argv.i
              i = i + 1
            end /* if */
            else
              zone.key = zone.key" 1"
            
            if argv.i<>"in" then rc=terminate("Unexpected word" argv.i)

            i = i + 1
            zone.key = zone.key" "argv.i
          end /* when obj */

          when argv.start="put" then do
            zone.ncmds = zone.ncmds + 1
            key = "cmd"zone.ncmds
            i = start + 1
            if vnumtype(argv.i)='o' then zone.key = "P "if_flag" "vnum(argv.i)
            else rc=terminate("Illegal object number" argv.i)
            zone.key = "P "if_flag" "argv.i

            i = i + 1
            if argv.i="max" then do
              i = i + 1
              zone.key = zone.key" "argv.i
              i = i + 1
            end /* if */
            else
              zone.key = zone.key" 1"
            
            if argv.i<>"in" then rc=terminate("Unexpected word" argv.i)

            if vnumtype(argv.i)='r' then zone.key = zone.key" "vnum(argv.i)
            else rc=terminate("Illegal room number" argv.i)
          end /* when put */

          when argv.start="remove" then do
            zone.ncmds = zone.ncmds + 1
            key = "cmd"zone.ncmds
            i = start + 1
            if vnumtype(argv.i)='o' then zone.key = "R "if_flag" "vnum(argv.i)
            else rc=terminate("Illegal object number" argv.i)

            i = i + 1
            if argv.i<>"from" then rc=terminate("Unexpected word" argv.i)

            i = i + 1
            if vnumtype(argv.i)='r' then zone.key = zone.key" "vnum(argv.i)
            else rc=terminate("Illegal room number" argv.i)
          end /* when remove */

          when argv.start="give" then do
            zone.ncmds = zone.ncmds + 1
            key = "cmd"zone.ncmds
            i = start + 1
            if vnumtype(argv.i)='o' then zone.key = "G "if_flag" "vnum(argv.i)
            else rc=terminate("Illegal object number" argv.i)

            i = i + 1
            select
              when argv.i="" then zone.key = zone.key" 1"

              when argv.i="max" then do
                i = i + 1
                if isint(argv.i) then zone.key = zone.key" "argv.i
                else rc=terminate("Illegal max number" argv.i)
              end  /* Do */

              otherwise
                rc=terminate("Unexpected word" argv.i)
            end /* select */
          end /* when give */

          when argv.start="equip" then do
            zone.ncmds = zone.ncmds + 1
            key = "cmd"zone.ncmds
            i = start + 1
            if vnumtype(argv.i)='o' then zone.key = "E "if_flag" "vnum(argv.i)
            else rc=terminate("Illegal object number" argv.i)

            i = i + 1
            if argv.i="max" then do
              i = i + 1
              if isint(argv.i) then zone.key = zone.key" "argv.i
              else rc=terminate("Illegal max number" argv.i)
              i = i + 1
            end /* if */
            else
              zone.key = zone.key" 1"

              j=wordpos(argv.i, "light rfinger lfinger neck1 neck2 body head "|| ,
                  "legs feet hands arms shield about waist rwrist lwrist "|| ,
                  "wield hold")-1
              if j>=0 then zone.key=zone.key" "j
              else rc=terminate("Unexpected word" argv.i)
          end /* when equip */

          when argv.start="door" then do
            zone.ncmds = zone.ncmds + 1
            key = "cmd"zone.ncmds
            i = start + 1
            if vnumtype(argv.i)='r' then zone.key = "D "if_flag" "vnum(argv.i)
            else rc=terminate("Illegal room number" argv.i)

            i = i + 1
            select
              when argv.i="n" then zone.key = zone.key" 0"
              when argv.i="e" then zone.key = zone.key" 1"
              when argv.i="s" then zone.key = zone.key" 2"
              when argv.i="w" then zone.key = zone.key" 3"
              when argv.i="u" then zone.key = zone.key" 4"
              when argv.i="d" then zone.key = zone.key" 5"
              otherwise
                rc=terminate("Invalid direction" argv.i)
            end /* select dir */

            i = i + 1
            select
              when argv.i="open" then zone.key = zone.key" 0"
              when argv.i="closed" then zone.key = zone.key" 1"
              when argv.i="locked" then zone.key = zone.key" 2"
              otherwise
                rc=terminate("Invalid state" argv.i)
            end /* select state */
          end /* when door */

          otherwise
            rc=terminate("Unknown command" argv.1)
      end /* select zone command */
    end /* if */

    do until line<>"" | lines(nwofile)=0
      line = getline()
    end /* do */
    rc=splitline(line)

  end /* while lines(nwofile)<>0 & line<>"end" */


  /* ------------------------------- */
  /* write zone */
  rc=fopen(zonfile, "a")
  rc=fprint(zonfile, "#"zone)
  rc=fprint(zonfile, zone.name"~")
  rc=fprint(zonfile, (zone*100+99)" "zone.lifespan" "zone.reset)

  do i=1 to zone.ncmds
    key = "cmd"i
    out = zone.key;  rc=fprint(zonfile, out)
  end

  rc=fclose(zonfile)
return ""


/* ------------------------------------------------------------------------- */
/* do_room(line) */
do_room: procedure expose argc argv. nwofile wldfile zonfile lineno zone debug cr lf newline system
/* local line n room. key dir i out */

  line=arg(1)
  n=word(line, 2)
  if vnumtype(n)<>'r' then rc=terminate("Illegal room number" n)
  n=vnum(n)
  if n%100<>zone then rc=terminate("Room number" n "out of range" zone*100"-"zone*100+99)

  room.=""
  room.sdesc="room" n
  room.ldesc="."
  room.ndesc=0               /* room.desc_key<n> and room.desc_text<n> hold the
                                actual data */
  room.flags="d"
  room.terrain=0
  room.dir=""

  do until line<>"" | lines(nwofile)=0
    line = getline()
  end
  rc=splitline(line)

  do while lines(nwofile)<>0 & line<>"endroom"

    select
      when argv.1="sdesc" then room.sdesc = wordwrap(argv.2)

      when argv.1="ldesc" then room.ldesc = wordwrap(argv.2)

      when argv.1="desc" then do
        room.ndesc = room.ndesc + 1

        rc=value("room.desc_key"room.ndesc, argv.2)
        rc=value("room.desc_text"room.ndesc, wordwrap(argv.3))
      end /* argv.1="desc" */

      when argv.1="flags" then do
        room.flags=""
        do i=2 to argc
          j=wordpos(argv.i, "dark death nomob indoors peaceful notrack "|| ,
              "nomagic tunnel private godroom")
          if j>0 then room.flags=room.flags||substr("abcdefghij", j, 1)
          else rc=terminate("Unknown flag" argv.i)
        end i

        if room.flags="" then room.flags=0

      end /* argv.1="flags" */

      when argv.1="terrain" then do
        j=wordpos(argv.2, "inside city field forest hills mountain water "|| ,
            "water_noswim underwater flying")-1
        if j>=0 then room.terrain=j
        else rc=terminate("Unknown terrain type" argv.2)
      end /* argv.1="terrain" */

      when wordpos(argv.1, "n e s w u d")<>0 then do
        dir = argv.1
        longdir = word("north east south west up down", wordpos(argv.1, "n e s w u d"))

        room.dir = room.dir||dir
        rc=value("room."dir"_to", "0")
        rc=value("room."dir"_type", "0")
        rc=value("room."dir"_key", "-1")
        rc=value("room."dir"_to", "-1")
        rc=value("room."dir"_look", "")

        rc=value("room."dir"_keyword", argv.2)
        i = 3
        if argv.i="to" then do
          i = i + 1
          if vnumtype(argv.i)='r' then rc=value("room."dir"_to", vnum(argv.i))
          else rc=terminate("Illegal room number" argv.i)

          i = i + 1
          if wordpos(argv.i, "closable lockable nopick")<>0 then do
            select /* argv.i */
              when argv.i = "closable" then rc=value("room."dir"_type", "1")
              when argv.i = "lockable" then rc=value("room."dir"_type", "1")
              when argv.i = "nopick" then rc=value("room."dir"_type", "2")
            end /* select */

            if argv.i <> "closable" then do
              i = i + 1
              if argv.i="key" then do
                i = i + 1
                if vnumtype(argv.i)='o' then rc=value("room."dir"_key", vnum(argv.i))
                else rc=terminate("Illegal object number" argv.i)
                i = i + 1
              end  /* if key */
            end /* if <> "closable" */
          end /* if type */
          
          if argv.i="reset" then do
            i = i + 1
            select
              when argv.i = "open" then rc=value("room."dir"_reset", 0)
              when argv.i = "closed" then rc=value("room."dir"_reset", 1)
              when argv.i = "locked" then rc=value("room."dir"_reset", 2)
              otherwise
                rc=terminate("Illegal reset state" argv.i)
            end /* select */
            i = i + 1
          end /* if reset */
        end /* if to */
        /* at this point, i = next arg to read */

        if argv.i="look" then do
           i = i + 1
           rc=value("room."dir"_look", wordwrap(argv.i))
        end /* if "look" */
      end /* when wordpos(argv.1, "n e s w u d")<>0 */

      otherwise
        rc=terminate("Unknown command" argv.1)
    end  /* select */

    do until line<>"" | lines(nwofile)=0
      line = getline()
    end /* do */
    rc=splitline(line)
  end /* while */

  /* ------------------------------- */
  /* write room */
  rc=fopen(wldfile, "a")
  rc=fprint(wldfile, "#"n)
  rc=fprint(wldfile, room.sdesc"~")
  rc=fprint(wldfile, room.ldesc)
  rc=fprint(wldfile, "~")

  /* make sure all indoors rooms have the indoors flag */
  if room.terrain="0" & pos("d", room.flags)=0 then do
    room.flags = room.flags"d"
    out = ""
    do i=1 to length("abcdefghij")
      c = substr("abcdefghij", i, 1)
      if pos(c, room.flags) then out = out||c
    end
    room.flags = out
  end /* if */

  rc=fprint(wldfile, (n%100)" "room.flags" "room.terrain)

  do i=1 to 6
    dir=substr("neswud", i, 1)
    if pos(dir, room.dir)<>0 then do

      rc=fprint(wldfile, "D"||(i-1))
      rc=fprint(wldfile, value("room."dir"_look"))
      rc=fprint(wldfile, "~")
      rc=fprint(wldfile, value("room."dir"_keyword")||"~")

      rc=fprint(wldfile, value("room."dir"_type")" "value("room."dir"_key")" "value("room."dir"_to"))

      if value("room."dir"_reset")<>"" then do
        rc=fopen(zonfile, "a")
        rc=fprint(zonfile, "D 0 "n" "||(i-1)" "value("room."dir"_reset"))
        rc=fclose(zonfile)
      end /* if reset */

    end /* if dir */
  end /* do */

  do i=1 to room.ndesc
    rc=fprint(wldfile, "E")
    rc=fprint(wldfile, value("room.desc_key"i)"~")
    rc=fprint(wldfile, value("room.desc_text"i))
    rc=fprint(wldfile, "~")
  end /* do */

  rc=fprint(wldfile, "S")

  rc=fclose(wldfile)
return ""


/* ------------------------------------------------------------------------- */
/* do_mob(line) */
do_mob: procedure expose argc argv. nwofile mobfile zonfile lineno zone xp_table. maxlevel debug cr lf newline system
/* local line n mob. zone. i out */

  line=arg(1)
  n=word(line, 2)
  if vnumtype(n)<>'m' then rc=terminate("Illegal mob number" n)
  n=vnum(n)
  if n%100<>zone then rc=terminate("Mob number" n "out of range" zone*100"-"zone*100+99)

  mob.=""
  mob.alias=""
  mob.sdesc=""
  mob.here=""
  mob.look=""
  mob.actflags=0             /* action flags */
  mob.affflags=0             /* affection flags */
  mob.align=0
  mob.level=1
  mob.thaco=""               /* if still "" at the end, set to 21-<level> */
  mob.ac=10
  mob.hp=""                  /* if still 0 at the end, set to <level>d8+0 */
  mob.dmg="1d6+0"
  mob.dmg_type=""            /* if set, mob is enhanced */
  mob.gold=0
  mob.xp=""                  /* if still "", set to a reasonable value by level */
  mob.loadpos=8              /* standing */
  mob.position=8             /* standing */
  mob.sex=0                  /* neuter */
  mob.type="S"               /* simple */
  mob.str=""                 /* if any of these are set, mob is enhanced */
  mob.stradd=""
  mob.int=""
  mob.wis=""
  mob.dex=""
  mob.con=""
  mob.cha=""

  do until line<>"" | lines(nwofile)=0
    line = getline()
  end
  rc=splitline(line)

  do while lines(nwofile)<>0 & line<>"endmob"

    select
      when argv.1="alias" then mob.alias = wordwrap(argv.2)

      when argv.1="sdesc" then mob.sdesc = wordwrap(argv.2)

      when argv.1="here" then mob.here = wordwrap(argv.2)

      when argv.1="look" then mob.look = wordwrap(argv.2)

      when argv.1="flags" then do
        mob.actflags=""
        mob.affflags=""
        do i=2 to argc
          j=wordpos(argv.i, "special sentinel scavenger aware aggressive "|| ,
              "stay_zone wimpy aggr_evil aggr_good aggr_neutral memory "|| ,
              "helper nocharm nosummon nosleep nobash noblind")
          if j>0 then mob.actflags=mob.actflags||substr("abcefghijklmnopqr", j, 1)
          else do
            j=wordpos(argv.i, "blind invis detect_align detect_invis "|| ,
                "detect_magic sense_life waterwalk sanctuary curse "|| ,
                "infravision protect_evil protect_good notrack sneak hide")
            if j>0 then mob.affflags=mob.affflags||substr("abcdefghjkmnpst", j, 1)
            else rc=terminate("Unknown flag" argv.i)
          end /* else */
        end i

        if mob.actflags="" then mob.actflags=0
        if mob.affflags="" then mob.affflags=0
      end /* argv.1="flags" */

      when argv.1="align" then do
        if isint(argv.2) then
          if argv.2>=-1000 & argv.2<=1000 then mob.align = argv.2
          else rc=terminate("Alignment" argv.2 "out of range -1000 to 1000")
        else rc=terminate("Illegal alignment value" n)
      end /* argv.1="align" */

      when argv.1="level" then do
        if isint(argv.2) then
          if argv.2>=0 & argv.2<=maxlevel then mob.level = argv.2
          else rc=terminate("Level" argv.2 "out of range 0 to" maxlevel)
        else rc=terminate("Illegal level value" n)
      end /* argv.1="level" */

      when argv.1="thaco" then do
        if isint(argv.2) then do
          if argv.2>=0 & argv.2<=20 then mob.thaco=argv.2
          else rc=terminate("THACO value" argv.2 "out of range 0 to 20")
        end /* if number */
        else do
          i=wordpos(argv.2, "fighter thief cleric mage")
          if i>0 then do
            if level="" then rc=terminate("Level must be defined before a calculated THACO.")

            select
              when i=1 then mob.thaco = 21 - level
              when i=2 then mob.thaco = 20 - ((level-1) % 2)
              when i=3 then mob.thaco = 20 - ((level-1) % 3 * 2)
              when i=4 then mob.thaco = 20 - ((level-1) % 3)
            end  /* select */

            if mob.thaco<0 then mob.thaco=0
            else if mob.thaco>20 then mob.thaco=20
          end /* if class */
          else
            rc=terminate("Illegal THACO value" argv.2)
        end /* if */

      end /* argv.1="thaco" */

      when argv.1="ac" then do
        if isint(argv.2) then do
          if argv.2>=-10 & argv.2<=10 then mob.ac = argv.2
          else rc=terminate("Armor class" argv.2 "out of range -10 to 10")
        end /* if */
        else
          rc=terminate("Illegal armor class" argv.2)
      end /* argv.1="ac" */

      when argv.1="hp" then do
        i=legaldice(argv.2)
        if i<>"" then mob.hp=i
        else rc=terminate("Illegal hit points value" argv.2)
      end /* argv.1="hp" */

      when argv.1="dmg" then do
        i=legaldice(argv.2)
        if i<>"" then mob.dmg=i
        else rc=terminate("Illegal damage value" argv.2)

        if argc=3 then do
          mob.type="E"
          mob.dmgtype=wordpos(argv.3, "hit sting whip slash bite bludgeon crush "|| ,
              "pound claw maul thrash pierce blast punch stab")-1
          if mob.dmgtype<0 then rc=terminate("Unknown damage type" argv.3)
        end /* if */
      end /* argv.1="dmg" */

      when argv.1="gold" then do
        if isint(argv.2) then do
          if argv.2>=0 then mob.gold=argv.2
          else rc=terminate("Illegal negative amount of gold" argv.2)
        end /* if */
        else
          rc=terminate("Illegal amount of gold" argv.2)
      end /* argv.1="gold" */

      when argv.1="xp" then do
        if isint(argv.2) then do
          if argv.2>=0 then mob.xp=argv.2
          else rc=terminate("Illegal negative xp value" argv.2)
        end /* if */
        else
          rc=terminate("Illegal xp value" argv.2)
      end /* argv.1="xp" */

      when argv.1="loadpos" then do
        select
          when argv.2="sleep" then  mob.loadpos=4
          when argv.2="rest" then mob.loadpos=5
          when argv.2="sit" then mob.loadpos=6
          when argv.2="stand" then mob.loadpos=8
          otherwise
            rc=terminate("Illegal loadpos value" argv.2)
        end /* select */
      end /* argv.1="loadpos" */

      when argv.1="position" then do
        select
          when argv.2="sleep" then  mob.position=4
          when argv.2="rest" then mob.position=5
          when argv.2="sit" then mob.position=6
          when argv.2="stand" then mob.position=8
          otherwise
            rc=terminate("Illegal position value" argv.2)
        end /* select */
      end /* argv.1="position" */

      when argv.1="sex" then do
        select
          when left(argv.2, 1)="n" then  mob.sex=0
          when left(argv.2, 1)="m" then  mob.sex=1
          when left(argv.2, 1)="f" then  mob.sex=2
          otherwise
            rc=terminate("Illegal sex" argv.2)
        end /* select */
      end /* argv.1="sex" */

      when argv.1="str" then do
        mob.type="E"
        if isint(argv.2) then do
          if argv.2>=3 & argv.2<=25 then mob.str=argv.2
          else rc=terminate("Strength value" argv.2 "out of range 3 to 25")
        end /* if */
        else rc=terminate("Illegal strength value" argv.2)
      end /* argv.1="str" */

      when argv.1="stradd" then do
        mob.type="E"
        if isint(argv.2) then do
          if argv.2>=1 & argv.2<=100 then mob.stradd=argv.2
          else rc=terminate("Strength add value" argv.2 "out of range 1 to 100")
        end /* if */
        else rc=terminate("Illegal strength add value" argv.2)
      end /* argv.1="stradd" */

      when argv.1="dex" then do
        mob.type="E"
        if isint(argv.2) then do
          if argv.2>=3 & argv.2<=25 then mob.dex=argv.2
          else rc=terminate("Dexterity value" argv.2 "out of range 3 to 25")
        end /* if */
        else rc=terminate("Illegal dexterity value" argv.2)
      end /* argv.1="dex" */

      when argv.1="con" then do
        mob.type="E"
        if isint(argv.2) then do
          if argv.2>=3 & argv.2<=25 then mob.con=argv.2
          else rc=terminate("Constitution value" argv.2 "out of range 3 to 25")
        end /* if */
        else rc=terminate("Illegal constitution value" argv.2)
      end /* argv.1="con" */

      when argv.1="int" then do
        mob.type="E"
        if isint(argv.2) then do
          if argv.2>=3 & argv.2<=25 then mob.int=argv.2
          else rc=terminate("Intelligence value" argv.2 "out of range 3 to 25")
        end /* if */
        else rc=terminate("Illegal intelligence value" argv.2)
      end /* argv.1="int" */

      when argv.1="wis" then do
        mob.type="E"
        if isint(argv.2) then do
          if argv.2>=3 & argv.2<=25 then mob.wis=argv.2
          else rc=terminate("Wisdom value" argv.2 "out of range 3 to 25")
        end /* if */
        else rc=terminate("Illegal wisdom value" argv.2)
      end /* argv.1="wis" */

      when argv.1="cha" then do
        mob.type="E"
        if isint(argv.2) then do
          if argv.2>=3 & argv.2<=25 then mob.cha=argv.2
          else rc=terminate("Charisma value" argv.2 "out of range 3 to 25")
        end /* if */
        else rc=terminate("Illegal charisma value" argv.2)
      end /* argv.1="cha" */

      when argv.1="create" then do
        zone.=""
        zone.max=1
        i=2                  /* next argv to read */
        if argv.2="max" then do
          i = i + 1
          if isint(argv.i) then max=argv.i
          else rc=terminate("Illegal max value" argv.i)
          i = i + 1
        end /* if max */

        if argv.i="in" then do
          i = i + 1
          if vnumtype(argv.i)='r' then zone.room=vnum(argv.i)
          else rc=terminate("Illegal room number" argv.i)
          i = i + 1
        end /* if in */
        else rc=terminate("Unexpected word" argv.i)

        /* write mob entry to zonfile */
        rc=fopen(zonfile, "a")
        rc=fprint(zonfile, "M 0 "n" "||(zone.max)" "zone.room)
        rc=fclose(zonfile)

        do while i<=argc
          zone.pos=wordpos(argv.i, "with light rfinger lfinger neck1 neck2 "|| ,
              "body head legs feet hands arms shield about waist rwrist "|| ,
              "lwrist wield hold")
          if zone.pos=0 then rc=terminate("Unexpected word" argv.i)
          i = i + 1

          zone.max=1
          if argv.i="max" then do
            i = i + 1
            if isint(argv.i) & argv.i>0 then zone.max=argv.i
            else rc=terminate("Illegal max value" argv.i)
            i = i + 1
          end /* if max */

          if vnumtype(argv.i)='o' then zone.obj=vnum(argv.i)
          else rc=terminate("Illegal object number" argv.i)
          i = i + 1

          /* write object entry to zonfile */
          if zone.pos=1 then "G 1 "zone.obj" "zone.max
          else out="E 1 "zone.obj" "zone.max" "||(zone.pos-2)
          rc=fprint(zonfile, out)
        end /* while i<=argc */
        rc=fclose(zonfile)

      end /* argv.1="create" */

      otherwise
        rc=terminate("Unknown command" argv.1)
    end  /* select */

    do until line<>"" | lines(nwofile)=0
      line = getline()
    end /* do */
    rc=splitline(line)
  end /* while */

  /* ------------------------------- */
  /* defaults */
  if mob.thaco="" then do
    mob.thaco = 21 - mob.level
    if mob.thaco<0 then mob.thaco=0
    else if mob.thaco>20 then mob.thaco=20
  end /* if mob.thaco */

  if mob.hp="" then mob.hp = mob.level"d8+0"

  if mob.xp="" then do
    i = mob.level
    mob.xp = xp_table.i * 3 % 20     /* xp awarded is divided by 3 in the source */
                                     /* for no apparent reason */
  end /* if mob.xp */

  /* ------------------------------- */
  /* write mob */
  rc=fopen(mobfile, "a")
  rc=fprint(mobfile, "#"n)
  rc=fprint(mobfile, mob.alias"~")
  rc=fprint(mobfile, mob.sdesc"~")
  rc=fprint(mobfile, mob.here)
  rc=fprint(mobfile, "~")
  rc=fprint(mobfile, mob.look)
  rc=fprint(mobfile, "~")
  rc=fprint(mobfile, mob.actflags" "mob.affflags" "mob.align" "mob.type)

  rc=fprint(mobfile, mob.level" "mob.thaco" "mob.ac" "mob.hp" "mob.dmg)
  rc=fprint(mobfile, mob.gold" "mob.xp)
  rc=fprint(mobfile, mob.loadpos" "mob.position" "mob.sex)

  if mob.type="E" then do
    if mob.dmgtype<>"" then do
      rc=fprint(mobfile, "BareHandAttack: "mob.dmgtype)
    end /* if dmgtype */

    if mob.str<>"" then do
      rc=fprint(mobfile, "Str: "mob.str)
    end /* if str */

    if mob.stradd<>"" then do
      rc=fprint(mobfile, "StrAdd: "mob.stradd)
    end /* if stradd */

    if mob.dex<>"" then do
      rc=fprint(mobfile, "Dex: "mob.dex)
    end /* if dex */

    if mob.con<>"" then do
      rc=fprint(mobfile, "Con: "mob.con)
    end /* if con */

    if mob.int<>"" then do
      rc=fprint(mobfile, "Int: "mob.int)
    end /* if int */

    if mob.wis<>"" then do
      rc=fprint(mobfile, "Wis: "mob.wis)
    end /* if wis */

    if mob.cha<>"" then do
      rc=fprint(mobfile, "Cha: "mob.cha)
    end /* if cha */

    rc=fprint(mobfile, "E")
  end /* if enhanced */

  rc=fclose(mobfile)
return ""


/* ------------------------------------------------------------------------- */
/* do_obj(line) */
do_obj: procedure expose argc argv. nwofile objfile lineno zone maxlevel debug cr lf newline system
/* local line n obj. zone. i j out */

  line=arg(1)
  n=word(line, 2)
  if vnumtype(n)<>'o' then rc=terminate("Illegal object number" n)
  n=vnum(n)
  if n%100<>zone then rc=terminate("Object number" n "out of range" zone*100"-"zone*100+99)

  obj.=""
  obj.alias=""
  obj.sdesc=""
  obj.here=""
  obj.action=""
  obj.ndesc=0                /* room.desc_key<n> and room.desc_text<n> hold the */
                             /* actual data */
  obj.weight=0
  obj.cost=0
  obj.rent=0
  obj.flags=0
  obj.wear=0
  obj.affect1=""
  obj.bonus1=0
  obj.affect2=""
  obj.bonus2=0
  obj.type=0
  obj.arg0=0
  obj.arg1=0
  obj.arg2=0
  obj.arg3=0


  do until line<>"" | lines(nwofile)=0
    line = getline()
  end
  rc=splitline(line)

  do while lines(nwofile)<>0 & line<>"endobj"
    select
      when argv.1="alias" then obj.alias = wordwrap(argv.2)

      when argv.1="sdesc" then obj.sdesc = wordwrap(argv.2)

      when argv.1="here" then obj.here = wordwrap(argv.2)

      when argv.1="action" then obj.action = wordwrap(argv.2)

      when argv.1="desc" then do
        obj.ndesc = obj.ndesc + 1

        rc=value("obj.desc_key"obj.ndesc, argv.2)
        rc=value("obj.desc_text"obj.ndesc, wordwrap(argv.3))
      end /* argv.1="desc" */

      when argv.1="weight" then do
        if isint(argv.2) & argv.2>=0 then obj.weight = argv.2
        else rc=terminate("Illegal weight value" n)
      end /* argv.1="weight" */

      when argv.1="cost" then do
        if isint(argv.2) & argv.2>=0 then obj.cost = argv.2
        else rc=terminate("Illegal cost $"n)
      end /* argv.1="weight" */

      when argv.1="rent" then do
        if isint(argv.2) & argv.2>=0 then obj.rent = argv.2
        else rc=terminate("Illegal rent $"n)
      end /* argv.1="weight" */

      when argv.1="flags" then do
        obj.flags=""
        do i=2 to argc
          j=wordpos(argv.i, "glow hum norent nodonate noinvis invis magic "|| ,
              "nodrop bless nogood noevil noneutral nomage nocleric "|| ,
              "nothief nowarrior nosell")
          if j>0 then obj.flags=obj.flags||substr("abcdefghijklmnopq", j, 1)
          else rc=terminate("Unknown flag" argv.i)
        end i
        if obj.flags="" then obj.flags=0
      end /* argv.1="flags" */

      when argv.1="wear" then do
        obj.wear=""
        do i=2 to argc
          j=wordpos(argv.i, "take finger neck body head legs feet hands arms shield about waist wrist wield hold")
          if j>0 then obj.wear=obj.wear||substr("abcdefghijklmno", j, 1)
          else rc=terminate("Unknown wear flag" argv.i)
        end i
        if obj.wear="" then obj.wear=0
      end /* argv.1="wear" */

      when argv.1="affect" then do
        i=wordpos(argv.2, "str dex int wis con cha age char_weight "|| ,
            "char_height mana hit move ac hitroll damroll saving_para "|| ,
            "saving_rod saving_petri saving_breath saving_spell")
        if i>0 then do
          obj.affect1=word(i, "1 2 3 4 5 6 9 10 11 12 13 14 17 18 19 20 21 22 23 24")
          if isint(argv.3) then obj.bonus1=argv.3
          else rc=terminate("Illegal affect bonus" argv.3)
        end /* if */
        else rc=terminate("Unknown affect flag" argv.2)

        if argv.4<>"" then do
          i=wordpos(argv.4, "str dex int wis con cha age char_weight "|| ,
              "char_height mana hit move ac hitroll damroll saving_para "|| ,
              "saving_rod saving_petri saving_breath saving_spell")
          if i>0 then do
            obj.affect1=word(i, "1 2 3 4 5 6 9 10 11 12 13 14 17 18 19 20 21 22 23 24")
            if isint(argv.5) then obj.bonus1=argv.5
            else rc=terminate("Illegal affect bonus" argv.5)
          end /* if */
          else rc=terminate("Unknown affect flag" argv.4)
        end /* if second affect */

      end /* argv.1="affect" */

      when argv.1="light" then do
        obj.type=1
        select
          when argv.2="eternal" then obj.arg2=-1
          when argv.2="burned_out" then obj.arg2=0
          when isint(argv.2) & argv.2>0 then obj.arg2=argv.2
          otherwise
            rc=terminate("Illegal light duration" argv.2)
        end /* select */
      end /* argv.1="light" */

      when argv.1="scroll" then do
        obj.type=2
        if isint(argv.2) then do
          if argv.2>=1 & argv.2<=maxlevel then obj.arg0=argv.2
          else rc=terminate("Spell level" argv.2 "out of range 1 to" maxlevel)
        end /* if */
        else rc=terminate("Illegal spell level" argv.2)

        obj.arg1=-1; obj.arg2=-1; obj.arg3=-1

        do i=3 to argc
          j=spellnum(argv.i)
          if j>0 then rc=value("obj.arg"||(i-2), j)
          else rc=terminate("Unknown spell" argv.i)
        end /* do */
      end /* argv.1="scroll" */

      when argv.1="wand" then do
        obj.type=3
        if isint(argv.2) then do
          if argv.2>=1 & argv.2<=maxlevel then obj.arg0=argv.2
          else rc=terminate("Wand level" argv.2 "out of range 1 to" maxlevel)
        end /* if */
        else rc=terminate("Illegal wand level" argv.2)

        if isint(argv.3) & argv.3>=1 then obj.arg1=argv.3
        else rc=terminate("Illegal charge capacity" argv.3)

        if isint(argv.4) & argv.4>=0 then obj.arg2=argv.4
        else rc=terminate("Illegal number of charges" argv.4)

        i=spellnum(argv.5)
        if i>0 then obj.arg3=i
        else rc=terminate("Unknown spell" argv.5)
      end /* argv.1="wand" */

      when argv.1="staff" then do
        obj.type=4
        if isint(argv.2) then do
          if argv.2>=1 & argv.2<=maxlevel then obj.arg0=argv.2
          else rc=terminate("Staff level" argv.2 "out of range 1 to" maxlevel)
        end /* if */
        else rc=terminate("Illegal staff level" argv.2)

        if isint(argv.3) & argv.3>=1 then obj.arg1=argv.3
        else rc=terminate("Illegal charge capacity" argv.3)

        if isint(argv.4) & argv.4>=0 then obj.arg2=argv.4
        else rc=terminate("Illegal number of charges" argv.4)

        i=spellnum(argv.5)
        if i>0 then obj.arg3=i
        else rc=terminate("Unknown spell" argv.5)
      end /* argv.1="staff" */

      when argv.1="weapon" then do
        obj.type=5
        i=legaldice(argv.2)
        if i<>"" then do
          obj.arg1=left(i, pos("d", i)-1)
          i=substr(i, pos("d", i)+1)
          obj.arg2=left(i, pos("+", i)-1)
        end /* if */
        else rc=terminate("Illegal weapon damage" argv.2)

        obj.arg3=wordpos(argv.3, "hit sting whip slash bite bludgeon crush "|| ,
            "pound claw maul thrash pierce blast punch stab")-1
        if obj.arg3<0 then rc=terminate("Unknown damage type" argv.3)
      end /* argv.1="weapon" */

      when argv.1="treasure" then do
        obj.type=8
      end /* argv.1="treasure" */

      when argv.1="armor" then do
        obj.type=9
        if isint(argv.2) then do
          if argv.2>=-10 & argv.2<=10 then obj.arg0=argv.2
          else rc=terminate("Armor class modifier" argv.2 "out of range -10 to 10")
        end /* if */
        else rc=terminate("Illegal armor class modifier" argv.2)
      end /* argv.1="armor" */

      when argv.1="potion" then do
        obj.type=10
        if isint(argv.2) then do
          if argv.2>=1 & argv.2<=maxlevel then obj.arg0=argv.2
          else rc=terminate("Potion level" argv.2 "out of range 1 to" maxlevel)
        end /* if */
        else rc=terminate("Illegal potion level" argv.2)

        obj.arg1=-1; obj.arg2=-1; obj.arg3=-1

        do i=3 to argc
          j=spellnum(argv.i)
          if j>0 then rc=value("obj.arg"||(i-2), j)
          else rc=terminate("Unknown spell" argv.i)
        end /* do */
      end /* argv.1="potion" */

      when argv.1="other" then do
        obj.type=12
      end /* argv.1="other" */

      when argv.1="trash" then do
        obj.type=13
      end /* argv.1="trash" */

      when argv.1="container" then do
        obj.type=15
        if isint(argv.2) & argv.2>0 then obj.arg0=argv.2
        else rc=terminate("Illegal capacity" argv.2)

        obj.arg1=0; obj.arg2=-1; obj.arg3=0

        if argc>2 then do
          select
            when argv.3="open" then    obj.arg1=1
            when argv.3="closed" then  obj.arg1=5
            when argv.3="locked" then  obj.arg1=13
            otherwise
              rc=terminate("Illegal lock state" argv.3)
          end /* select */

          if argv.4="nopick" then do
            obj.arg1=obj.arg1+2
            i=5
          end
          else i=4

          if argv.i="key" then do
            i = i + 1
            if vnumtype(argv.i)='o' then obj.arg2=vnum(argv.i)
            else rc=terminate("Illegal key number" argv.i)
          end /* if */
        end /* if argc>2 */

      end /* argv.1="container" */

      when argv.1="note" then do
        obj.type=16
      end /* argv.1="note" */

      when argv.1="drink" then do
        obj.type=17
        i=wordpos(argv.2, "water beer wine ale darkale whisky dew cola "|| ,
            "local slime milk tea coffee blood saltwater clearwater "|| ,
            "lemonade firebreath")-1
        if i>=16 then i = i - 10   /* replaced lemonade and firebreath with */
                                   /* dew and cola */
        if i>=0 then obj.arg2=i
        else rc=terminate("Unknown drink type" argv.2)

        if isint(argv.3) & argv.2>0 then obj.arg0=argv.3
        else rc=terminate("Illegal drink container capacity" argv.3)

        if isint(argv.4) & argv.3>0 then obj.arg1=argv.4
        else rc=terminate("Illegal drink container quantity" argv.4)

        if argv.5="poison" then obj.arg3=1
        else if argc>4 then rc=terminate("Unexpected word" argv.5)
      end /* argv.1="drink" */

      when argv.1="key" then do
        obj.type=18
      end /* argv.1="key" */

      when argv.1="food" then do
        obj.type=19
        if isint(argv.2) & argv.2>=0 then obj.arg0=argv.2
        else rc=terminate("Illegal food value" argv.2)

        if argc>2 then do
          if argv.3="poison" then obj.arg3=1
          else rc=terminate("Unexpected word" argv.3)
        end /* if */
      end /* argv.1="food" */

      when argv.1="money" then do
        obj.type=20
        if isint(argv.2) and argv.1>0 then obj.arg0=argv.2
        else rc=terminate("Illegal number of coins $"argv.2)

        if argc>2 then rc=terminate("Unexpected word" argv.3)
      end /* argv.1="money" */

      when argv.1="pen" then do
        obj.type=21
      end /* argv.1="pen" */

      when argv.1="boat" then do
        obj.type=22
      end /* argv.1="boat" */

      when argv.1="fountain" then do
        obj.type=23
         i=wordpos(argv.2, "water beer wine ale darkale whisky dew cola "|| ,
            "local slime milk tea coffee blood saltwater clearwater "|| ,
            "lemonade firebreath")-1
        if i>=16 then i = i - 10   /* replaced lemonade and firebreath with */
                                   /* dew and cola */
        if i>=0 then obj.arg2=i
        else rc=terminate("Unknown drink type" argv.2)

        if isint(argv.3) & argv.2>0 then obj.arg0=argv.3
        else rc=terminate("Illegal drink container capacity" argv.3)

        if isint(argv.4) & argv.3>0 then obj.arg1=argv.4
        else rc=terminate("Illegal drink container quantity" argv.4)

        if argv.5="poison" then obj.arg3=1
        else if argc>4 then rc=terminate("Unexpected word" argv.5)
      end /* argv.1="fountain" */

      otherwise
        rc=terminate("Unknown command" argv.1)
    end  /* select */

    do until line<>"" | lines(nwofile)=0
      line = getline()
    end /* do */
    rc=splitline(line)
  end /* while */

  /* ------------------------------- */
  /* write obj */
  rc=fopen(objfile, "a")
  rc=fprint(objfile, "#"n)
  rc=fprint(objfile, obj.alias"~")
  rc=fprint(objfile, obj.sdesc"~")
  rc=fprint(objfile, obj.here"~")
  rc=fprint(objfile, obj.action"~")
  rc=fprint(objfile, obj.type" "obj.flags" "obj.wear)
  rc=fprint(objfile, obj.arg0" "obj.arg1" "obj.arg2" "obj.arg3)
  rc=fprint(objfile, obj.weight" "obj.cost" "obj.rent)

  if obj.affect1<>"" then do
    rc=fprint(objfile, "A")
    rc=fprint(objfile, obj.affect1" "obj.bonus1)
  end /* if */
  if obj.affect2<>"" then do
    rc=fprint(objfile, "A")
    rc=fprint(objfile, obj.affect2" "obj.bonus2)
  end /* if */

  do i=1 to obj.ndesc
    rc=fprint(objfile, "E")
    rc=fprint(objfile, value("obj.desc_key"i)"~")
    rc=fprint(objfile, value("obj.desc_text"i))
    rc=fprint(objfile, "~")
  end /* do */

  rc=fclose(objfile)
return ""


/* ------------------------------------------------------------------------- */
/* do_shop(line) */
do_shop: procedure expose argc argv. nwofile shpfile lineno zone debug cr lf newline system
/* local line n shop. i */
  line=arg(1)
  n=word(line, 2)
  if vnumtype(n)<>'s' then rc=terminate("Illegal shop number" n)
  n=vnum(n)
  if n%100<>zone then rc=terminate("Shop number" n "out of range" zone*100"-"zone*100+99)

  shop.=""
  shop.keeper=""
  shop.rooms=""
  shop.sell_profit="1.0"
  shop.buy_profit="0.5"
  shop.produces=""
  shop.nbuy=0                /* shop.buy<n> holds the actual buy$ */
  shop.buy_nexist="%s You don't have that!"
  shop.sell_nexist="%s I don't have that!"
  shop.dont_buy="%s I don't buy that kind of thing."
  shop.shop_nsf="%s I cannot afford it."
  shop.player_nsf="%s I do not accept IOUs!"
  shop.bought="%s I'll give you %d coins."
  shop.sold="%s That'll be %d coins."
  shop.hour1=0
  shop.hour2=0
  shop.hour3=0
  shop.hour4=0
  shop.temper=2
  shop.flags=0
  shop.with_who=0

  do until line<>"" | lines(nwofile)=0
    line = getline()
  end
  rc=splitline(line)

  do while lines(nwofile)<>0 & line<>"endshop"

    select
      when argv.1="shopkeeper" then do
        if vnumtype(argv.2)='m' then shop.keeper = vnum(argv.2)
        else rc=terminate("Illegal shopkeeper number" argv.2)
      end /* argv.1="shopkeeper" */

      when argv.1="room" then do
        shop.rooms=""
        do i=2 to argc
          if vnumtype(argv.i)='r' then shop.rooms=shop.rooms" "vnum(argv.i)
          else rc=terminate("Illegal room number" argv.i)
        end i
      end /* argv.1="room" */

      when argv.1="hours" then do
        if argc<>3 & argc<>5 then rc=terminate("Illegal hours command")

        do i=2 to argc
          if isint(argv.i) then do
            if argv.i>=0 & argv.i<=23 then rc=value("shop.hour"||(i-1), argv.i)
            else rc=terminate("Hour" argv.i "out of range 0 to 23")
          end /* if */
          else rc=terminate("Illegal hour" argv.i)
        end i
      end /* argv.1="hours" */

      when argv.1="profit" then do
        if datatype(argv.2, 'N') & argv.2>=1.0 then shop.sell_profit=argv.2
        else rc=terminate("Illegal sales profit" argv.2)

        if datatype(argv.3, 'N') & argv.3<=1.0 then shop.buy_profit=argv.3
        else rc=terminate("Illegal purchasing profit" argv.3)
      end /* argv.1="profit" */

      when argv.1="produces" then do
        shop.produces=""
        do i=2 to argc
          if vnumtype(argv.i)='o' then shop.produces=vnum(argv.i)
          else rc=terminate("Illegal object number" argv.i)
        end i
      end /* argv.1="produces" */

      when argv.1="buys" then do
        shop.nbuy=0
        do i=2 to argc
          rc=value("shop.buy"||(i-1), argv.i)
          /* FIXME - add some error checking */
          shop.nbuy = shop.nbuy + 1
        end i
      end /* argv.1="buys" */

      when argv.1="buy_nexist" then shop.buy_nexist=argv.2
      when argv.1="sell_nexist" then shop.sell_nexist=argv.2
      when argv.1="dont_buy" then shop.dont_buy=argv.2
      when argv.1="shop_nsf" then shop.shop_nsf=argv.2
      when argv.1="player_nsf" then shop.player_nsf=argv.2
      when argv.1="bought" then shop.bought=argv.2
      when argv.1="sold" then shop.sold=argv.2

      when argv.1="flags" then do
        shop.flags=""
        shop.with_who=""
        do i=2 to argc
          select
            when argv.i="temper0" then shop.temper = 0
            when argv.i="temper1" then shop.temper = 0
            when argv.i="fight" then shop.flags = shop.flags||"a"
            when argv.i="uses_bank" then shop.flags = shop.flags||"b"
            when argv.i="nogood" then shop.with_who = shop.with_who||"a"
            when argv.i="noevil" then shop.with_who = shop.with_who||"b"
            when argv.i="noneutral" then shop.with_who = shop.with_who||"c"
            when argv.i="nomage" then shop.with_who = shop.with_who||"d"
            when argv.i="nocleric" then shop.with_who = shop.with_who||"e"
            when argv.i="nothief" then shop.with_who = shop.with_who||"f"
            when argv.i="nowarrior" then shop.with_who = shop.with_who||"g"
            otherwise
              rc=terminate("Unknown flag" argv.i)
          end /* select */
        end i
        if shop.flags="" then shop.flags=0
        if shop.with_who="" then shop.with_who=0
      end /* argv.1="flags" */

      otherwise
        rc=terminate("Unknown command" argv.1)
    end  /* select */

    do until line<>"" | lines(nwofile)=0
      line = getline()
    end /* do */
    rc=splitline(line)
  end /* while */

  /* ------------------------------- */
  /* set up shop file if it doesn't already exist */
  if \fexists(shpfile) then do
    rc=fopen(shpfile, "rw")
    rc=fprint(shpfile, "CircleMUD v3.0 Shop File~")
  end /* if shpfile exists */
  else rc=fopen(shpfile, "a")

  /* ------------------------------- */
  /* write shop */
  rc=fprint(shpfile, "#"n"~")
  do i=1 to words(shop.produces)
    rc=fprint(shpfile, word(shop.produces, i))
  end i
  rc=fprint(shpfile, "-1")

  rc=fprint(shpfile, shop.sell_profit)
  rc=fprint(shpfile, shop.buy_profit)

  do i=1 to shop.nbuy
    rc=fprint(shpfile, value("shop.buy"i))
  end i
  rc=fprint(shpfile, "-1")

  rc=fprint(shpfile, shop.buy_nexist"~")
  rc=fprint(shpfile, shop.sell_nexist"~")
  rc=fprint(shpfile, shop.dont_buy"~")
  rc=fprint(shpfile, shop.shop_nsf"~")
  rc=fprint(shpfile, shop.player_nsf"~")
  rc=fprint(shpfile, shop.bought"~")
  rc=fprint(shpfile, shop.sold"~")

  rc=fprint(shpfile, shop.temper)
  rc=fprint(shpfile, shop.flags)
  rc=fprint(shpfile, shop.keeper)
  rc=fprint(shpfile, shop.with_who)

  do i=1 to words(shop.rooms)
    rc=fprint(shpfile, word(shop.rooms, i))
  end i
  rc=fprint(shpfile, "-1")

  rc=fprint(shpfile, shop.hour1)
  rc=fprint(shpfile, shop.hour2)
  rc=fprint(shpfile, shop.hour3)
  rc=fprint(shpfile, shop.hour4)

  rc=fclose(shpfile)
return ""

/* ------------------------------------------------------------------------- */
/* printerr(msg) */
/* prints msg to standard error */
printerr: procedure expose system
  if system="os2" then return lineout("stderr:", arg(1))
return lineout("<stderr>", arg(1))


/* ------------------------------------------------------------------------- */
/* terminate(msg) */
/* prints a standardized error message and where it happened, then dies */
terminate:
  rc=printerr(arg(1) "at line" lineno "in file" nwofile)
  rc=printerr(line)
  exit -1


/* ------------------------------------------------------------------------- */
/* fexists(file) */
/* returns 1 if file exists, 0 if it does not. */
fexists: procedure expose system
  if system="os2" then return (stream(arg(1), "C", "query exists")<>"")
return (chars(arg(1))>0)


/* ------------------------------------------------------------------------- */
/* fopen(file, mode) */
/* tries to open file for "r" read, "w" write, "rw" read/write, or "a" append */
fopen: procedure expose system
/* local file mode */
  file=arg(1)
  mode=arg(2)

  if system="os2" then do
    select
    when mode="r" then return stream(file, "C", "open read")
    when mode="w" then return stream(file, "C", "open write")
    when mode="rw" then return stream(file, "C", "open")
    when mode="a" then do
      if \fexists(arg(1)) then
        rc=stream(arg(1), "C", "open write")
      else do
        rc=stream(arg(1), "C", "open")
        rc=stream(arg(1), "C", "seek <0")
      end /* else */
    end /* when */
    otherwise
      rc=printerr("Unknown fopen mode" mode)
      exit -1
    end /* select */
  end /* if OS/2 */
  else do
    select
    when mode="r" then return stream(file, "C", "read")
    when mode="w" then return stream(file, "C", "write")
    when mode="rw" then return stream(file, "C", "update")
    when mode="a" then return stream(file, "C", "append")
    otherwise
      rc=printerr("Unknown fopen mode" mode)
      exit -1
    end /* select */
  end /* else */
/* fopen: */


/* ------------------------------------------------------------------------- */
/* fclose(file) */
fclose:
return stream(arg(1), "C", "close")


/* ------------------------------------------------------------------------- */
/* isint(n) */
/* verifies that 'n' is an integer, returns 1 or 0 */
isint:
  if datatype(arg(1), 'N') then
    if arg(1)=trunc(arg(1)) then return 1
return 0


/* ------------------------------------------------------------------------- */
/* vnumtype(str) */
/* Determines if a string is a valid virtual number (r99, o99, etc). */
/* Returns the first letter if it is, or "" if it isn't. */
vnumtype: procedure
/* local str */
  str = arg(1)
  if length(str)<2 | pos(left(str, 1), "zroms")=0 then return ""
  if \ isint(substr(str, 2)) then return ""
return left(str, 1)


/* ------------------------------------------------------------------------- */
/* vnum(str) */
/* Returns the value of the vnum, does no error checking, so call vnumtype */
/* first. */
vnum:
return substr(arg(1), 2)

/* ------------------------------------------------------------------------- */
/* legaldice(str) */
/* returns a legal dice format if one can be made out of str, or "" if it's */
/* not a legal dice value */
legaldice: procedure expose debug
/* local i str l c n s m */
/* <n>d<s>+<m> */
  if debug then trace o

  str=arg(1); l=length(str)
  n="";  s="";  m=""

  i = 1
  do while i<=l
    c = substr(str, i, 1)
    if c>='0' & c<='9' then n=n||c
    else do
      if c='d' then do
        i = i + 1
        leave
      end /* if */
      else do
        trace r
        return ""
      end /* else */
    end /* else */
    i = i + 1
  end /* do */

  if i>l then do
    if debug then trace r
    return n"d1+0"
  end /* if */

  do while i<=l
    c = substr(str, i, 1)
    if c>='0' & c<='9' then s=s||c
    else do
      if c='+' | c='-' then do
        i = i + 1
        leave
      end /* if */
      else do
        if debug then trace r
        return ""
      end /* else */
    end /* else */
    i = i + 1
  end /* do */

  if i>l then do
    if debug then trace r
    return n"d"s"+0"
  end /* if */

  m=c
  do while i<=l
    c = substr(str, i, 1)
    if c>='0' & c<='9' then m=m||c
    else do
      if debug then trace r
      return ""
    end /* else */
    i = i + 1
  end /* do */

  if debug then trace r
return n"d"s"+"m

/* ------------------------------------------------------------------------- */
/* spellnum(str) */
/* verifies that a spell number or name is legitimate, and returns the spell */
/* number, or 0 if it is illegal. */
spellnum: procedure expose spell. debug
/* local i str l */
  if debug then trace o

  str=arg(1)
  l=length(str)

  if l=0 then do
    if debug then trace r
    return 0
  end
  if isint(str) then do
    if debug then trace r
    return str
  end

  do i=1 to 51
    if str=left(spell.i, l) then do
      if debug then trace r
      return i
    end
  end /* do */

  do i=201 to 206
    if str=left(spell.i, l) then do
      if debug then trace r
      return i
    end
  end /* do */
  if debug then trace r
return 0


/* ------------------------------------------------------------------------- */
/* getline() */
/* reads a line from nwofile, stripping out comments and leading and trailing */
/* spaces, and then merging lines ending in \ */
getline: procedure expose argc argv. nwofile lineno debug cr lf newline
/* local line cont in */
  if debug then trace o
  cont=1
  line=""

  do until lines(nwofile)=0 | \ cont
    in=linein(nwofile,,1)
    lineno = lineno + 1
    if debug then rc=printout(lineno": "in)

    /* strip comments */
    i = pos("//", in)
    if i>0 then in=left(in, i-1)

    /* strip spaces */
    in = strip(in, "B")

    line = line||in

    /* continuation? */
    if right(line, 1)="\" then do
      line = left(line, length(line)-1)
      cont = 1
    end /* if */
    else cont = 0

  end /* until lines(nwofile)=0 | \ cont */
  if debug then trace r
return line


/* ------------------------------------------------------------------------- */
/* splitline(string)
   splitline breaks a string up into arguments in the argv. stem - argc is
   the number of args found, argv.1...argv.argc are the actual words.  Quotes
   and backslash escapes can be used, including \n (newline) and \e (escape).
*/
splitline: procedure expose argc argv. debug cr lf newline
/* local str, wrd, len, mode, mnormal, mquote, mend, i, c */
  if debug then trace o

  mnormal = 0
  mquote = 1
  mend = 2

  parse arg str
  len = length(str)
  mode = mnormal

  argc = 1
  argv. = ""

  /* speed up the simplest case */
  if pos('"', str)=0 & pos('\', str)=0 then do
    argc = words(str)
    do i = 1 to argc
      argv.i = word(str, i)
    end
    if debug then do
      do i=1 to argc
        rc=printerr(i": '"argv.i"'")
      end
      trace r
    end /* if debug */
    return argc
  end /* if */

  i = 1
  do while i<=len
    c = substr(str, i, 1)
    do while i<=len & (c=' ' | c=d2c(9))
      i = i + 1
      c = substr(str, i, 1)
    end /* while (c=' ' | c=d2c(9)) */

    if i>len then leave

    /* scan arg */
    do while i<=len & mode<>mend

      select /* mode */
        when mode=mnormal then do
          c = substr(str, i, 1)
          select /* c */

            when c='\' then do
              i = i + 1
              if i<=len then do
                c = substr(str, i, 1)
                select /* c */
                   when c='n' then c = lf
                   when c='e' then c = d2c(27)
                end  /* select */

                argv.argc = argv.argc||c
                i = i + 1
              end
              else argv.argc = argv.argc||'\'
            end /* when c='\' */

            when c='"' then do
              i = i + 1
              mode = mquote
            end /* when c='"' */

            when c=' ' then do
              i = i + 1
              mode = mend
            end /* when c=' ' */

            otherwise
              argv.argc = argv.argc||c
              i = i + 1

          end /* select c */
        end /* when mode=mnormal */

        when mode=mquote then do
          c = substr(str, i, 1)

          select /* select c */

            when c='\' then do
              i = i + 1
              if i<=len then do
                argv.argc = argv.argc||substr(str, i, 1)
                i = i + 1
              end
              else argv.argc = argv.argc||'\'
            end /* when c='\' */

            when c='"' then do
              i = i + 1
              mode = mnormal
            end /* when c='"' */

            otherwise
              argv.argc = argv.argc||c
              i = i + 1

          end /* select c */

        end /* when mode=mquote */
      end /* select mode */
    end /* while mode<>mend */

    argc = argc + 1
    mode = mnormal

  end /* do while i<=len */

  argc = argc - 1

  if debug then do
    do i=1 to argc
      rc=printerr(i": '"argv.i"'")
    end
    trace r
  end /* if debug */
return argc


/* ------------------------------------------------------------------------- */
/* wordwrap(str)
   wordwarp inserts newlines in long lines (more than 79 characters), either by
   replacing a space or by inserting a newline after a dash.  If all else
   fails, the line will be broken.  The processed string with lf's inserted will
   be returned.
*/
wordwrap: procedure expose debug cr lf newline
/* local str l nlines line. i s c */
  if debug then trace o

  str = arg(1)
  l = length(str)
  nlines = 0
  line. = ""

  do while str<>""
    nlines = nlines + 1
    s = "";  sl = 0;  c = ""

    do i=1 to l
      c = substr(str, i, 1)
      if c=lf then do
        s = s||c;  sl = sl + 1
        leave i
      end /* if c=lf */

      s = s||c;  sl = sl + 1
      if sl=80 then leave i
    end i
    /* s now contains a line 80 chars or less, ready to be word-wrapped */

    if substr(s, sl, 1)=lf then do
      line.nlines = left(s, sl-1)
    end /* if last char=lf */

    else if l<=80 then do
      line.nlines = str
      str = ""; l = 0
      leave
    end /* else if l<=80 */

    /* otherwise, find the last space in the line and replace it with an lf */
    else do
      i = lastpos(' ', s)
      if i>0 then do
        s = left(s, i-1)
        line.nlines = s
        sl = i  /* note that the space is cropped */
      end /* if space was found */
      else do
        i = lastpos('-', s)
        if i>0 then do
          s = left(s, i-1)
          line.nlines = s
          sl = i  /* note that the - is not cropped */
        end /* if - was found */
        else line.nlines = s

      end /* else space was not found */

    end /* else */

    str = substr(str, sl+1)
    l = l - sl

  end /* while str<>"" */

  do i=1 to nlines
    str = str||line.i||lf
  end i
  str = left(str, length(str)-1)  /* strip off final lf */

  if debug then trace r
return str


/* ------------------------------------------------------------------------- */
/* fprint(file, str)
   fprint writes a multi-line string to "file", or if not specified, to stdout.
   This is an inefficient way to do this, but it does not noticeably slow NWO,
   so for now it stays.
*/
fprint: procedure expose debug cr lf newline
/* local str file l i c */

  file = arg(1)
  if file="" then file="stdout:"

  str = arg(2)
  l = length(str)
  s = ""

  do i=1 to l
    c = substr(str, i, 1)
    if c=lf then do
      rc=lineout(file, s)
      s = ""
    end /* if */
    else s = s||c
  end i
  rc=lineout(file, s)
return ""

